@@ -56,14 +56,20 @@ class Node(_common.ListMixin, resource.Resource):
         is_maintenance='maintenance',
     )
 
-    # The conductor_group field introduced in 1.46 (Rocky).
-    _max_microversion = '1.46'
+    # The allocation_uuid field introduced in 1.52 (Stein).
+    _max_microversion = '1.52'
 
     # Properties
+    #: The UUID of the allocation associated with this node. Added in API
+    #: microversion 1.52.
+    allocation_id = resource.Body("allocation_uuid")
     #: The UUID of the chassis associated wit this node. Can be empty or None.
     chassis_id = resource.Body("chassis_uuid")
     #: The current clean step.
     clean_step = resource.Body("clean_step")
+    #: Hostname of the conductor currently handling this ndoe. Added in API
+    # microversion 1.49.
+    conductor = resource.Body("conductor")
     #: Conductor group this node is managed by. Added in API microversion 1.46.
     conductor_group = resource.Body("conductor_group")
     #: Timestamp at which the node was last updated.
@@ -91,11 +97,16 @@ class Node(_common.ListMixin, resource.Resource):
     instance_info = resource.Body("instance_info")
     #: UUID of the nova instance associated with this node.
     instance_id = resource.Body("instance_uuid")
+    #: Override enabling of automated cleaning. Added in API microversion 1.47.
+    is_automated_clean_enabled = resource.Body("automated_clean", type=bool)
     #: Whether console access is enabled on this node.
     is_console_enabled = resource.Body("console_enabled", type=bool)
     #: Whether node is currently in "maintenance mode". Nodes put into
     #: maintenance mode are removed from the available resource pool.
     is_maintenance = resource.Body("maintenance", type=bool)
+    # Whether the node is protected from undeploying. Added in API microversion
+    # 1.48.
+    is_protected = resource.Body("protected", type=bool)
     #: Any error from the most recent transaction that started but failed to
     #: finish.
     last_error = resource.Body("last_error")
@@ -118,6 +129,8 @@ class Node(_common.ListMixin, resource.Resource):
     #: Physical characteristics of the node. Content populated by the service
     #: during inspection.
     properties = resource.Body("properties", type=dict)
+    # The reason why this node is protected. Added in API microversion 1.48.
+    protected_reason = resource.Body("protected_reason")
     #: The current provisioning state of the node.
     provision_state = resource.Body("provision_state")
     #: The current RAID configuration of the node.
@@ -447,7 +460,42 @@ class Node(_common.ListMixin, resource.Resource):
                 "the last error is %(error)s" %
                 {'node': self.id, 'error': self.last_error})
 
-    def attach_vif(self, session, vif_id):
+    # TODO(dtantsur): waiting for power state
+    def set_power_state(self, session, target):
+        """Run an action modifying this node's power state.
+
+        This call is asynchronous, it will return success as soon as the Bare
+        Metal service acknowledges the request.
+
+        :param session: The session to use for making this request.
+        :type session: :class:`~keystoneauth1.adapter.Adapter`
+        :param target: Target power state, e.g. "rebooting", "power on".
+            See the Bare Metal service documentation for available actions.
+        """
+        session = self._get_session(session)
+
+        if target.startswith("soft "):
+            version = '1.27'
+        else:
+            version = None
+
+        version = utils.pick_microversion(session, version)
+
+        # TODO(dtantsur): server timeout support
+        body = {'target': target}
+
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'states', 'power')
+        response = session.put(
+            request.url, json=body,
+            headers=request.headers, microversion=version,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+        msg = ("Failed to set power state for bare metal node {node} "
+               "to {target}".format(node=self.id, target=target))
+        exceptions.raise_from_response(response, error_message=msg)
+
+    def attach_vif(self, session, vif_id, retry_on_conflict=True):
         """Attach a VIF to the node.
 
         The exact form of the VIF ID depends on the network interface used by
@@ -458,6 +506,10 @@ class Node(_common.ListMixin, resource.Resource):
         :param session: The session to use for making this request.
         :type session: :class:`~keystoneauth1.adapter.Adapter`
         :param string vif_id: Backend-specific VIF ID.
+        :param retry_on_conflict: Whether to retry HTTP CONFLICT errors.
+            This can happen when either the VIF is already used on a node or
+            the node is locked. Since the latter happens more often, the
+            default value is True.
         :return: ``None``
         :raises: :exc:`~openstack.exceptions.NotSupported` if the server
             does not support the VIF API.
@@ -470,12 +522,13 @@ class Node(_common.ListMixin, resource.Resource):
         request = self._prepare_request(requires_id=True)
         request.url = utils.urljoin(request.url, 'vifs')
         body = {'id': vif_id}
+        retriable_status_codes = _common.RETRIABLE_STATUS_CODES
+        if not retry_on_conflict:
+            retriable_status_codes = set(retriable_status_codes) - {409}
         response = session.post(
             request.url, json=body,
             headers=request.headers, microversion=version,
-            # NOTE(dtantsur): do not retry CONFLICT, it's a valid status code
-            # in this API when the VIF is already attached to another node.
-            retriable_status_codes=[503])
+            retriable_status_codes=retriable_status_codes)
 
         msg = ("Failed to attach VIF {vif} to bare metal node {node}"
                .format(node=self.id, vif=vif_id))
