@@ -12,6 +12,8 @@
 
 import os
 
+from openstack.load_balancer.v2 import flavor
+from openstack.load_balancer.v2 import flavor_profile
 from openstack.load_balancer.v2 import health_monitor
 from openstack.load_balancer.v2 import l7_policy
 from openstack.load_balancer.v2 import l7_rule
@@ -33,6 +35,9 @@ class TestLoadBalancer(base.BaseFunctionalTest):
     POOL_ID = None
     VIP_SUBNET_ID = None
     PROJECT_ID = None
+    FLAVOR_PROFILE_ID = None
+    FLAVOR_ID = None
+    AMPHORA_ID = None
     PROTOCOL = 'HTTP'
     PROTOCOL_PORT = 80
     LB_ALGORITHM = 'ROUND_ROBIN'
@@ -47,6 +52,9 @@ class TestLoadBalancer(base.BaseFunctionalTest):
     COMPARE_TYPE = 'CONTAINS'
     L7RULE_TYPE = 'HOST_NAME'
     L7RULE_VALUE = 'example'
+    AMPHORA = 'amphora'
+    FLAVOR_DATA = '{"loadbalancer_topology": "SINGLE"}'
+    DESCRIPTION = 'Test description'
 
     @classmethod
     def setUpClass(cls):
@@ -70,6 +78,8 @@ class TestLoadBalancer(base.BaseFunctionalTest):
         self.MEMBER_NAME = self.getUniqueString()
         self.POOL_NAME = self.getUniqueString()
         self.UPDATE_NAME = self.getUniqueString()
+        self.FLAVOR_PROFILE_NAME = self.getUniqueString()
+        self.FLAVOR_NAME = self.getUniqueString()
         subnets = list(self.conn.network.subnets())
         self.VIP_SUBNET_ID = subnets[0].id
         self.PROJECT_ID = self.conn.session.get_project_id()
@@ -81,6 +91,21 @@ class TestLoadBalancer(base.BaseFunctionalTest):
                                 'member': 100})
         assert isinstance(test_quota, quota.Quota)
         self.assertEqual(self.PROJECT_ID, test_quota.id)
+
+        test_profile = self.conn.load_balancer.create_flavor_profile(
+            name=self.FLAVOR_PROFILE_NAME, provider_name=self.AMPHORA,
+            flavor_data=self.FLAVOR_DATA)
+        assert isinstance(test_profile, flavor_profile.FlavorProfile)
+        self.assertEqual(self.FLAVOR_PROFILE_NAME, test_profile.name)
+        self.FLAVOR_PROFILE_ID = test_profile.id
+
+        test_flavor = self.conn.load_balancer.create_flavor(
+            name=self.FLAVOR_NAME, flavor_profile_id=self.FLAVOR_PROFILE_ID,
+            is_enabled=True, description=self.DESCRIPTION)
+        assert isinstance(test_flavor, flavor.Flavor)
+        self.assertEqual(self.FLAVOR_NAME, test_flavor.name)
+        self.FLAVOR_ID = test_flavor.id
+
         test_lb = self.conn.load_balancer.create_load_balancer(
             name=self.LB_NAME, vip_subnet_id=self.VIP_SUBNET_ID,
             project_id=self.PROJECT_ID)
@@ -93,6 +118,10 @@ class TestLoadBalancer(base.BaseFunctionalTest):
             wait=self._wait_for_timeout)
         self.LB_ID = test_lb.id
 
+        amphorae = self.conn.load_balancer.amphorae(loadbalancer_id=self.LB_ID)
+        for amp in amphorae:
+            self.AMPHORA_ID = amp.id
+
         test_listener = self.conn.load_balancer.create_listener(
             name=self.LISTENER_NAME, protocol=self.PROTOCOL,
             protocol_port=self.PROTOCOL_PORT, loadbalancer_id=self.LB_ID)
@@ -190,6 +219,12 @@ class TestLoadBalancer(base.BaseFunctionalTest):
             self.LB_ID, ignore_missing=False)
         super(TestLoadBalancer, self).tearDown()
 
+        self.conn.load_balancer.delete_flavor(self.FLAVOR_ID,
+                                              ignore_missing=False)
+
+        self.conn.load_balancer.delete_flavor_profile(self.FLAVOR_PROFILE_ID,
+                                                      ignore_missing=False)
+
     def test_lb_find(self):
         test_lb = self.conn.load_balancer.find_load_balancer(self.LB_NAME)
         self.assertEqual(self.LB_ID, test_lb.id)
@@ -200,6 +235,15 @@ class TestLoadBalancer(base.BaseFunctionalTest):
         self.assertEqual(self.LB_ID, test_lb.id)
         self.assertEqual(self.VIP_SUBNET_ID, test_lb.vip_subnet_id)
 
+    def test_lb_get_stats(self):
+        test_lb_stats = self.conn.load_balancer.get_load_balancer_statistics(
+            self.LB_ID)
+        self.assertEqual(0, test_lb_stats.active_connections)
+        self.assertEqual(0, test_lb_stats.bytes_in)
+        self.assertEqual(0, test_lb_stats.bytes_out)
+        self.assertEqual(0, test_lb_stats.request_errors)
+        self.assertEqual(0, test_lb_stats.total_connections)
+
     def test_lb_list(self):
         names = [lb.name for lb in self.conn.load_balancer.load_balancers()]
         self.assertIn(self.LB_NAME, names)
@@ -219,6 +263,13 @@ class TestLoadBalancer(base.BaseFunctionalTest):
         test_lb = self.conn.load_balancer.get_load_balancer(self.LB_ID)
         self.assertEqual(self.LB_NAME, test_lb.name)
 
+    def test_lb_failover(self):
+        self.conn.load_balancer.failover_load_balancer(self.LB_ID)
+        self.conn.load_balancer.wait_for_load_balancer(
+            self.LB_ID, wait=self._wait_for_timeout)
+        test_lb = self.conn.load_balancer.get_load_balancer(self.LB_ID)
+        self.assertEqual(self.LB_NAME, test_lb.name)
+
     def test_listener_find(self):
         test_listener = self.conn.load_balancer.find_listener(
             self.LISTENER_NAME)
@@ -231,6 +282,15 @@ class TestLoadBalancer(base.BaseFunctionalTest):
         self.assertEqual(self.PROTOCOL, test_listener.protocol)
         self.assertEqual(self.PROTOCOL_PORT, test_listener.protocol_port)
 
+    def test_listener_get_stats(self):
+        test_listener_stats = self.conn.load_balancer.get_listener_statistics(
+            self.LISTENER_ID)
+        self.assertEqual(0, test_listener_stats.active_connections)
+        self.assertEqual(0, test_listener_stats.bytes_in)
+        self.assertEqual(0, test_listener_stats.bytes_out)
+        self.assertEqual(0, test_listener_stats.request_errors)
+        self.assertEqual(0, test_listener_stats.total_connections)
+
     def test_listener_list(self):
         names = [ls.name for ls in self.conn.load_balancer.listeners()]
         self.assertIn(self.LISTENER_NAME, names)
@@ -449,3 +509,94 @@ class TestLoadBalancer(base.BaseFunctionalTest):
 
     def test_default_quota(self):
         self.conn.load_balancer.get_quota_default()
+
+    def test_providers(self):
+        providers = self.conn.load_balancer.providers()
+        # Make sure our default provider is in the list
+        self.assertTrue(
+            any(prov['name'] == self.AMPHORA for prov in providers))
+
+    def test_provider_flavor_capabilities(self):
+        capabilities = self.conn.load_balancer.provider_flavor_capabilities(
+            self.AMPHORA)
+        # Make sure a known capability is in the default provider
+        self.assertTrue(any(
+            cap['name'] == 'loadbalancer_topology' for cap in capabilities))
+
+    def test_flavor_profile_find(self):
+        test_profile = self.conn.load_balancer.find_flavor_profile(
+            self.FLAVOR_PROFILE_NAME)
+        self.assertEqual(self.FLAVOR_PROFILE_ID, test_profile.id)
+
+    def test_flavor_profile_get(self):
+        test_flavor_profile = self.conn.load_balancer.get_flavor_profile(
+            self.FLAVOR_PROFILE_ID)
+        self.assertEqual(self.FLAVOR_PROFILE_NAME, test_flavor_profile.name)
+        self.assertEqual(self.FLAVOR_PROFILE_ID, test_flavor_profile.id)
+        self.assertEqual(self.AMPHORA, test_flavor_profile.provider_name)
+        self.assertEqual(self.FLAVOR_DATA, test_flavor_profile.flavor_data)
+
+    def test_flavor_profile_list(self):
+        names = [fv.name for fv in self.conn.load_balancer.flavor_profiles()]
+        self.assertIn(self.FLAVOR_PROFILE_NAME, names)
+
+    def test_flavor_profile_update(self):
+        self.conn.load_balancer.update_flavor_profile(
+            self.FLAVOR_PROFILE_ID, name=self.UPDATE_NAME)
+        test_flavor_profile = self.conn.load_balancer.get_flavor_profile(
+            self.FLAVOR_PROFILE_ID)
+        self.assertEqual(self.UPDATE_NAME, test_flavor_profile.name)
+
+        self.conn.load_balancer.update_flavor_profile(
+            self.FLAVOR_PROFILE_ID, name=self.FLAVOR_PROFILE_NAME)
+        test_flavor_profile = self.conn.load_balancer.get_flavor_profile(
+            self.FLAVOR_PROFILE_ID)
+        self.assertEqual(self.FLAVOR_PROFILE_NAME, test_flavor_profile.name)
+
+    def test_flavor_find(self):
+        test_flavor = self.conn.load_balancer.find_flavor(self.FLAVOR_NAME)
+        self.assertEqual(self.FLAVOR_ID, test_flavor.id)
+
+    def test_flavor_get(self):
+        test_flavor = self.conn.load_balancer.get_flavor(self.FLAVOR_ID)
+        self.assertEqual(self.FLAVOR_NAME, test_flavor.name)
+        self.assertEqual(self.FLAVOR_ID, test_flavor.id)
+        self.assertEqual(self.DESCRIPTION, test_flavor.description)
+        self.assertEqual(self.FLAVOR_PROFILE_ID, test_flavor.flavor_profile_id)
+
+    def test_flavor_list(self):
+        names = [fv.name for fv in self.conn.load_balancer.flavors()]
+        self.assertIn(self.FLAVOR_NAME, names)
+
+    def test_flavor_update(self):
+        self.conn.load_balancer.update_flavor(
+            self.FLAVOR_ID, name=self.UPDATE_NAME)
+        test_flavor = self.conn.load_balancer.get_flavor(self.FLAVOR_ID)
+        self.assertEqual(self.UPDATE_NAME, test_flavor.name)
+
+        self.conn.load_balancer.update_flavor(
+            self.FLAVOR_ID, name=self.FLAVOR_NAME)
+        test_flavor = self.conn.load_balancer.get_flavor(self.FLAVOR_ID)
+        self.assertEqual(self.FLAVOR_NAME, test_flavor.name)
+
+    def test_amphora_list(self):
+        amp_ids = [amp.id for amp in self.conn.load_balancer.amphorae()]
+        self.assertIn(self.AMPHORA_ID, amp_ids)
+
+    def test_amphora_find(self):
+        test_amphora = self.conn.load_balancer.find_amphora(self.AMPHORA_ID)
+        self.assertEqual(self.AMPHORA_ID, test_amphora.id)
+
+    def test_amphora_get(self):
+        test_amphora = self.conn.load_balancer.get_amphora(self.AMPHORA_ID)
+        self.assertEqual(self.AMPHORA_ID, test_amphora.id)
+
+    def test_amphora_configure(self):
+        self.conn.load_balancer.configure_amphora(self.AMPHORA_ID)
+        test_amp = self.conn.load_balancer.get_amphora(self.AMPHORA_ID)
+        self.assertEqual(self.AMPHORA_ID, test_amp.id)
+
+    def test_amphora_failover(self):
+        self.conn.load_balancer.failover_amphora(self.AMPHORA_ID)
+        test_amp = self.conn.load_balancer.get_amphora(self.AMPHORA_ID)
+        self.assertEqual(self.AMPHORA_ID, test_amp.id)
