@@ -22,6 +22,7 @@ import uuid
 from testscenarios import load_tests_apply_scenarios as load_tests  # noqa
 
 from openstack.cloud import exc
+from openstack import exceptions
 from openstack.tests import fakes
 from openstack.tests.unit import base
 
@@ -50,7 +51,8 @@ class TestBaremetalNode(base.IronicTestCase):
 
         machines = self.cloud.list_machines()
         self.assertEqual(2, len(machines))
-        self.assertEqual(self.fake_baremetal_node, machines[0])
+        self.assertSubdict(self.fake_baremetal_node, machines[0])
+        self.assertSubdict(fake_baremetal_two, machines[1])
         self.assert_calls()
 
     def test_get_machine(self):
@@ -90,13 +92,64 @@ class TestBaremetalNode(base.IronicTestCase):
                          self.fake_baremetal_node['uuid'])
         self.assert_calls()
 
-    def test_validate_node(self):
+    def test_validate_machine(self):
         # NOTE(TheJulia): Note: These are only the interfaces
-        # that are validated, and both must be true for an
+        # that are validated, and all must be true for an
         # exception to not be raised.
-        # This should be fixed at some point, as some interfaces
-        # are important in some cases and should be validated,
-        # such as storage.
+        validate_return = {
+            'boot': {
+                'result': True,
+            },
+            'deploy': {
+                'result': True,
+            },
+            'management': {
+                'result': True,
+            },
+            'power': {
+                'result': True,
+            },
+            'foo': {
+                'result': False,
+            }}
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     resource='nodes',
+                     append=[self.fake_baremetal_node['uuid'],
+                             'validate']),
+                 json=validate_return),
+        ])
+        self.cloud.validate_machine(self.fake_baremetal_node['uuid'])
+
+        self.assert_calls()
+
+    def test_validate_machine_not_for_deploy(self):
+        validate_return = {
+            'deploy': {
+                'result': False,
+                'reason': 'Not ready',
+            },
+            'power': {
+                'result': True,
+            },
+            'foo': {
+                'result': False,
+            }}
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     resource='nodes',
+                     append=[self.fake_baremetal_node['uuid'],
+                             'validate']),
+                 json=validate_return),
+        ])
+        self.cloud.validate_machine(self.fake_baremetal_node['uuid'],
+                                    for_deploy=False)
+
+        self.assert_calls()
+
+    def test_deprecated_validate_node(self):
         validate_return = {
             'deploy': {
                 'result': True,
@@ -119,36 +172,33 @@ class TestBaremetalNode(base.IronicTestCase):
 
         self.assert_calls()
 
-    # FIXME(TheJulia): So, this doesn't presently fail, but should fail.
-    # Placing the test here, so we can sort out the issue in the actual
-    # method later.
-    # def test_validate_node_raises_exception(self):
-    #    validate_return = {
-    #        'deploy': {
-    #            'result': False,
-    #            'reason': 'error!',
-    #        },
-    #        'power': {
-    #            'result': False,
-    #            'reason': 'meow!',
-    #        },
-    #        'foo': {
-    #            'result': True
-    #        }}
-    #    self.register_uris([
-    #        dict(method='GET',
-    #             uri=self.get_mock_url(
-    #                 resource='nodes',
-    #                 append=[self.fake_baremetal_node['uuid'],
-    #                         'validate']),
-    #             json=validate_return),
-    #    ])
-    #    self.assertRaises(
-    #        Exception,
-    #        self.cloud.validate_node,
-    #        self.fake_baremetal_node['uuid'])
-    #
-    #    self.assert_calls()
+    def test_validate_machine_raises_exception(self):
+        validate_return = {
+            'deploy': {
+                'result': False,
+                'reason': 'error!',
+            },
+            'power': {
+                'result': True,
+                'reason': None,
+            },
+            'foo': {
+                'result': True
+            }}
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     resource='nodes',
+                     append=[self.fake_baremetal_node['uuid'],
+                             'validate']),
+                 json=validate_return),
+        ])
+        self.assertRaises(
+            exceptions.ValidationException,
+            self.cloud.validate_machine,
+            self.fake_baremetal_node['uuid'])
+
+        self.assert_calls()
 
     def test_patch_machine(self):
         test_patch = [{
@@ -156,6 +206,11 @@ class TestBaremetalNode(base.IronicTestCase):
             'path': '/instance_info'}]
         self.fake_baremetal_node['instance_info'] = {}
         self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     resource='nodes',
+                     append=[self.fake_baremetal_node['uuid']]),
+                 json=self.fake_baremetal_node),
             dict(method='PATCH',
                  uri=self.get_mock_url(
                      resource='nodes',
@@ -247,6 +302,8 @@ class TestBaremetalNode(base.IronicTestCase):
         self.fake_baremetal_node['provision_state'] = 'inspect failed'
         self.fake_baremetal_node['last_error'] = 'kaboom!'
         inspecting_node['provision_state'] = 'inspecting'
+        finished_node = self.fake_baremetal_node.copy()
+        finished_node['provision_state'] = 'manageable'
         self.register_uris([
             dict(
                 method='GET',
@@ -266,7 +323,13 @@ class TestBaremetalNode(base.IronicTestCase):
                 uri=self.get_mock_url(
                     resource='nodes',
                     append=[self.fake_baremetal_node['uuid']]),
-                json=inspecting_node)
+                json=inspecting_node),
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid']]),
+                json=finished_node),
         ])
 
         self.cloud.inspect_machine(self.fake_baremetal_node['uuid'])
@@ -297,6 +360,12 @@ class TestBaremetalNode(base.IronicTestCase):
                     resource='nodes',
                     append=[self.fake_baremetal_node['uuid']]),
                 json=inspecting_node),
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid']]),
+                json=self.fake_baremetal_node),
         ])
         self.cloud.inspect_machine(self.fake_baremetal_node['uuid'])
 
@@ -908,7 +977,8 @@ class TestBaremetalNode(base.IronicTestCase):
                 self.fake_baremetal_node,
                 timeout=1))
 
-        self.assertEqual(0, len(self.adapter.request_history))
+        # NOTE(dtantsur): service discovery apparently requires 3 calls
+        self.assertEqual(3, len(self.adapter.request_history))
 
     def test_wait_for_baremetal_node_lock_timeout(self):
         self.fake_baremetal_node['reservation'] = 'conductor0'
@@ -1093,7 +1163,7 @@ class TestBaremetalNode(base.IronicTestCase):
         # this code is never used in the current code path.
         return_value = self.cloud.register_machine(nics, **node_to_post)
 
-        self.assertDictEqual(available_node, return_value)
+        self.assertSubdict(available_node, return_value)
         self.assert_calls()
 
     def test_register_machine_enroll_wait(self):
@@ -1174,7 +1244,7 @@ class TestBaremetalNode(base.IronicTestCase):
         return_value = self.cloud.register_machine(
             nics, wait=True, **node_to_post)
 
-        self.assertDictEqual(available_node, return_value)
+        self.assertSubdict(available_node, return_value)
         self.assert_calls()
 
     def test_register_machine_enroll_failure(self):
@@ -1605,10 +1675,92 @@ class TestBaremetalNode(base.IronicTestCase):
         update_dict = self.cloud.update_machine(
             self.fake_baremetal_node['uuid'])
         self.assertIsNone(update_dict['changes'])
-        self.assertDictEqual(self.fake_baremetal_node, update_dict['node'])
+        self.assertSubdict(self.fake_baremetal_node, update_dict['node'])
 
         self.assert_calls()
 
+    def test_attach_port_to_machine(self):
+        vif_id = '953ccbee-e854-450f-95fe-fe5e40d611ec'
+        self.register_uris([
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid']]),
+                json=self.fake_baremetal_node),
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    service_type='network',
+                    resource='ports.json',
+                    base_url_append='v2.0'),
+                json={'ports': [{'id': vif_id}]}),
+            dict(
+                method='POST',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid'], 'vifs'])),
+        ])
+        self.cloud.attach_port_to_machine(self.fake_baremetal_node['uuid'],
+                                          vif_id)
+        self.assert_calls()
+
+    def test_detach_port_from_machine(self):
+        vif_id = '953ccbee-e854-450f-95fe-fe5e40d611ec'
+        self.register_uris([
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid']]),
+                json=self.fake_baremetal_node),
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    service_type='network',
+                    resource='ports.json',
+                    base_url_append='v2.0'),
+                json={'ports': [{'id': vif_id}]}),
+            dict(
+                method='DELETE',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid'], 'vifs',
+                            vif_id])),
+        ])
+        self.cloud.detach_port_from_machine(self.fake_baremetal_node['uuid'],
+                                            vif_id)
+        self.assert_calls()
+
+    def test_list_ports_attached_to_machine(self):
+        vif_id = '953ccbee-e854-450f-95fe-fe5e40d611ec'
+        fake_port = {'id': vif_id, 'name': 'test'}
+        self.register_uris([
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid']]),
+                json=self.fake_baremetal_node),
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid'], 'vifs']),
+                json={'vifs': [{'id': vif_id}]}),
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    service_type='network',
+                    resource='ports.json',
+                    base_url_append='v2.0'),
+                json={'ports': [fake_port]}),
+        ])
+        res = self.cloud.list_ports_attached_to_machine(
+            self.fake_baremetal_node['uuid'])
+        self.assert_calls()
+        self.assertEqual([fake_port], res)
+
 
 class TestUpdateMachinePatch(base.IronicTestCase):
     # NOTE(TheJulia): As appears, and mordred describes,
@@ -1629,7 +1781,7 @@ class TestUpdateMachinePatch(base.IronicTestCase):
             self.fake_baremetal_node[self.field_name] = None
         value_to_send = self.fake_baremetal_node[self.field_name]
         if self.changed:
-            value_to_send = 'meow'
+            value_to_send = self.new_value
         uris = [dict(
             method='GET',
             uri=self.get_mock_url(
@@ -1641,7 +1793,7 @@ class TestUpdateMachinePatch(base.IronicTestCase):
             test_patch = [{
                 'op': 'replace',
                 'path': '/' + self.field_name,
-                'value': 'meow'}]
+                'value': value_to_send}]
             uris.append(
                 dict(
                     method='PATCH',
@@ -1657,28 +1809,37 @@ class TestUpdateMachinePatch(base.IronicTestCase):
         update_dict = self.cloud.update_machine(
             self.fake_baremetal_node['uuid'], **call_args)
 
-        if not self.changed:
+        if self.changed:
+            self.assertEqual(['/' + self.field_name], update_dict['changes'])
+        else:
             self.assertIsNone(update_dict['changes'])
-        self.assertDictEqual(self.fake_baremetal_node, update_dict['node'])
+        self.assertSubdict(self.fake_baremetal_node, update_dict['node'])
 
         self.assert_calls()
 
     scenarios = [
         ('chassis_uuid', dict(field_name='chassis_uuid', changed=False)),
         ('chassis_uuid_changed',
-         dict(field_name='chassis_uuid', changed=True)),
+         dict(field_name='chassis_uuid', changed=True,
+              new_value='meow')),
         ('driver', dict(field_name='driver', changed=False)),
-        ('driver_changed', dict(field_name='driver', changed=True)),
+        ('driver_changed', dict(field_name='driver', changed=True,
+                                new_value='meow')),
         ('driver_info', dict(field_name='driver_info', changed=False)),
-        ('driver_info_changed', dict(field_name='driver_info', changed=True)),
+        ('driver_info_changed', dict(field_name='driver_info', changed=True,
+                                     new_value={'cat': 'meow'})),
         ('instance_info', dict(field_name='instance_info', changed=False)),
         ('instance_info_changed',
-         dict(field_name='instance_info', changed=True)),
+         dict(field_name='instance_info', changed=True,
+              new_value={'cat': 'meow'})),
         ('instance_uuid', dict(field_name='instance_uuid', changed=False)),
         ('instance_uuid_changed',
-         dict(field_name='instance_uuid', changed=True)),
+         dict(field_name='instance_uuid', changed=True,
+              new_value='meow')),
         ('name', dict(field_name='name', changed=False)),
-        ('name_changed', dict(field_name='name', changed=True)),
+        ('name_changed', dict(field_name='name', changed=True,
+                              new_value='meow')),
         ('properties', dict(field_name='properties', changed=False)),
-        ('properties_changed', dict(field_name='properties', changed=True))
+        ('properties_changed', dict(field_name='properties', changed=True,
+                                    new_value={'cat': 'meow'}))
     ]
